/******************************************************************************
 * Boot Loader v1.0                                                           *
 * di Valerio Di Gregorio                                                     *
 *                                                                            *
 * Boot Loader v1.0 carica da shell di FreeDOS il "Nucleo di Calcolatori      *
 * Elettronici v1.0". Questo progetto  conforme alle "Multiboot              *
 * Specification version 0.6.95".                                             *
 ******************************************************************************/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mboot.h"
#include "bldr16.h"
#include "vgainit.h"

void init_multiboot_info(multiboot_info_t* mbi) {
    mbi->flags=0;
    mbi->mem_lower=0;
    mbi->mem_upper=0;
    mbi->boot_device=0;
    mbi->cmdline=0;
    mbi->mods_count=0;
    mbi->mods_addr=0;
    mbi->u.aout_sym.tabsize=0;
    mbi->u.aout_sym.strsize=0;
    mbi->u.aout_sym.addr=0;
    mbi->u.aout_sym.reserved=0;
    mbi->mmap_length=0;
    mbi->mmap_addr=0;
    mbi->framebuffer_addr.L=0;
    mbi->framebuffer_addr.H=0;
    mbi->framebuffer_pitch=0;
    mbi->framebuffer_width=0;
    mbi->framebuffer_height=0;
    mbi->framebuffer_bpp=0;
}

/* Determina le dimensioni della memoria */
int get_mem_info(unsigned long* lower, unsigned long* upper) {
    unsigned long temp = get_mem_upper();
    if(temp<0) return 0;
    *lower = get_mem_lower();
    *upper = temp >> 10;
    return 1;
}

/* Stampa un messaggio di errore ricorrente */
void printerror(const char* argv0) {
    fprintf(stderr, "Errore! Controllare la sintassi del comando. Per ulteriori informazioni:\n%s -h\n", argv0);
    return;
}

extern unsigned char ch_getline(char c, int l);
extern int ch_getmaxline();
void dump_bm(const char* fname) {
  FILE* f;
  int nl, c, l;

  f = fopen(fname, "w");
  
  if (!f) {
    perror(fname);
    return;
  }

  nl = ch_getmaxline();

  for (c = 0; c < 128; c++) {
    fprintf(f, "// %d %c\n", c, (isgraph(c) ? c : ' '));
    for (l = 0; l < nl; l++) {
      fprintf(f, "\t0x%02x,\n", ch_getline(c, l));
    }
    fprintf(f, "\n");
  }
  fclose(f);
}
      
  


int main(int argc, char* argv[]) {
    int svgamode=1; 		/*variabile per decidere la modalit grafica*/
    int dumpbitmap=0;
    
    int nextargv=1;             /* Prossimo argomento della linea di comando da processare */
    char* temp;
    
    ImgCoff_go32 bldr32;        /* Immagine dell'eseguibile COFF relativo alla parte a 32 bit del Boot Loader */
    ImgCoff_go32 kernel;        /* Immagine dell'eseguibile COFF relativo al kernel */

    multiboot_info_t mbi;       /* Informazioni multiboot da passare al nucleo */
    init_multiboot_info(&mbi);  /* Inizializzazione di mbi */

    /* sintassi: bootldr -h */
    if (argc == 2 && strcmp(argv[nextargv],"-h") == 0) {
	printf("Carica il nucleo di Calcolatori Elettronici.\n\n");
	printf("%s [opzioni] <modulo> [argomenti...]\n\n", argv[0]);
	printf("opzioni:\n");
	printf("-h\tVisualizza questo help.\n");
	printf("-t\tnucleo in modalit testo\n");
	return EXIT_SUCCESS;
    }

    if (nextargv >= argc) {
	printerror(argv[0]);
	return EXIT_FAILURE;
    }

    if (strcmp(argv[nextargv],"-t")==0){ /*argomento opzionale per rimanere in modalit testo*/
      svgamode=0;
      nextargv++;
    }  else if (strcmp(argv[nextargv], "-b") == 0) {
      dumpbitmap = 1;
      nextargv++;
    }



    if (!dumpbitmap) {
      /* gli argomenti rimanenti formano la cmdline per il nucleo */
      if (nextargv + 1 < argc) {
          int i, len = 0;
          char *p;

          mbi.flags |= 0x00000004;                        /* Impostazione del flag */
          for (i = nextargv + 1; i < argc; i++)
          	len += strlen(argv[i]) + 1;
          mbi.cmdline = (unsigned long)malloc(len * sizeof(char));
          p = (char *)mbi.cmdline;
          for (i = nextargv + 1; i < argc; i++) {
          	char *q = argv[i];
          	while (*q) *p++ = *q++;
          	*p++ = ' ';
          }
          *(p - 1) = '\0';
          mbi.cmdline = vaddr(get_ds(), mbi.cmdline);     /* Calcolo dell'indirizzo lineare */
      }

      /* Impostazione delle informazioni sulla memoria */
      if (get_mem_info(&mbi.mem_lower, &mbi.mem_upper))
          mbi.flags|=0x00000001;    /* Impostazione del flag */

      /* Caricamento degli eseguibili in memoria */
      load_coff(BLDR32_NAME, &bldr32);     
      load_coff(argv[nextargv], &kernel);
    }

    if(svgamode) {
	vgainit();
	mbi.flags |= MULTIBOOT_INFO_FRAMEBUFFER_INFO;
	mbi.framebuffer_addr.L = lfb;
	mbi.framebuffer_width = lfb_w;
	mbi.framebuffer_height = lfb_h;
	mbi.framebuffer_bpp = 8;
    } else
	textinit();

    /* Chiamata a launch16 */

    if (dumpbitmap) {
       dump_bm(argv[nextargv]);
    } else {   
      launch(&kernel, &mbi, bldr32.entry_point.v);
    }

    if(svgamode)
	vgaclose();
    else
	textclose();
}

