; Boot Loader v1.0
; di Valerio Di Gregorio
;
; Boot Loader v1.0 carica da shell di FreeDOS il "Nucleo di Calcolatori
; Elettronici v1.0". Questo progetto  conforme alle "Multiboot
; Specification version 0.6.95".
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SEGMENT _DATA PRIVATE CLASS=DATA

save_ds:
dw 0000h
save_ss:
dw 0000h        
save_esp:
dd 00000000h

; Descrittore della IDT
idtr:
dw 0000h
dd 00000000h


; Descrittore della GDT
gdtr:
gdt_limit: dw gdt_end-gdt       ; Limite della GDT
gdt_base : dd 00000000h         ; Indirizzo base della GDT (inizializzato dinamicamente)

sData   equ     18h                     ; dData - GDTbeg
real_idtr  dw      0FFFFh                       ; Limit and base of real mode IDT
        dd      0

; Tabella GDT
gdt:

;SELETTORE 00h (Selettore nullo)
dd 00000000h
dd 00000000h


;SELETTORE 08h (Segmento codice: eseguibile o lettura)
;Base = 00000000h;
;Limit = 0FFFFFh;
;P = 1
;DPL = 00
;Type = 1101b
;A = 0
;G = 1
;D = 1
;X = 0
;U = 0

dw 0FFFFh      ; Limit[15:0]
dw 0000h       ; Base[15:0]
db 00h         ; Base[23:16]
db 9Ah         ; P, DPL, Type, A
db 0CFh        ; G, D, X, U, Limit[19:16]
db 00h         ; Base[31:24]


;SELETTORE 10h (Segmento dati: lettura e scrittura)
;Base = 00000000h;
;Limit = 0FFFFFh;
;P = 1
;DPL = 00
;Type = 1001b
;A = 0
;G = 1
;D = 1
;X = 0
;U = 0

dw 0FFFFh      ; Limit[15:0]
dw 0000h       ; Base[15:0]
db 00h         ; Base[23:16]
db 92h         ; P, DPL, Type, A
db 0CFh        ; G, D, X, U, Limit[19:16]
db 00h         ; Base[31:24]

gdt_end:

mem_buf:
baseAddrLow:    dd 00000000h
baseAddrHigh:   dd 00000000h
lengthLow:      dd 00000000h
lengthHigh:     dd 00000000h
type:           dd 00000000h
mem_buf_end:

                            
                                                    

SEGMENT _TEXT PUBLIC CLASS=CODE USE16




global _launch, _get_mem_lower, _get_mem_upper, _get_ds

; Funzione: empty_8042
; Attende che il buffer della tastiera si svuoti.
; Parametri: nessuno
; Risultato: nessuno
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                
empty_8042:           push      ax

                      in        al, 64h                 ; Lettura del registro STR
                      test      al, 02h                 ; Controllo del bit FO
                      jnz       empty_8042              ; Attesa di FO == 0

                      pop       ax
                      ret


; Funzione: enable_a20
; Abilita la linea A20
; Parametri: nessuno
; Risultato: nessuno
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

enable_a20:           push      ax

                      call      empty_8042
                      mov       al, 0D1h                ; Comando per la scrittura di un byte di comando
                      out       64h, al                 ; Scrittura del comando in CTR

                      call      empty_8042
                      mov       al,0DFh                 ; Byte di comando
                      out       60h, al                 ; Scrittura del byte di comando in TBR

                      call      empty_8042

                      pop       ax

                      ret


; Funzione: disable_a20
; Disabilita la linea A20
; Parametri: nessuno
; Risultato: nessuno
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

disable_a20:          push      ax

                      call      empty_8042
                      mov       al, 0D1h                ; Comando per la scrittura di un byte di comando
                      out       64h, al                 ; Scrittura del comando in CTR

                      call      empty_8042
                      mov       al,0DDh                 ; Byte di comando
                      out       60h, al                 ; Scrittura del byte di comando in TBR

                      call      empty_8042

                      pop       ax

                      ret

; Funzione: launch16(char* path, multiboot_info_t* mbi, void* launch32);
; Abilita la modalit protetta e lancia la parte a 32 bit del Boot Loader.
; Parametri: &kernel = puntatore alla struttura immagine del nucleo;
;            &mbi = indirizzo della struttura Multiboot Information da passare al nucleo;
;            &bldr32 = entry point della parte a 32 bit del boot loader;
; Risultato: nessuno (non ritorna al chiamante)
;
; Sitazione dello stack successivamente al prologo:
;
;       |--------------|
;     0 |      bp      |  <- bp
;       |--------------|
;     2 |      ip      |
;       |--------------|
;     4 |   &kernel    |
;       |--------------|
;     6 |     &mbi     |
;       |--------------|
;     8 |   &bldr32    |
;       |--------------|
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

_launch:              

		      push      bp
                      mov       bp, sp
		      pushf
                      push      bx
                      push      cx
                      push      dx
                      push      si
                      push      di
		      push	es
		      push      fs
		      push      gs
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;; Passaggio alla modalit protetta ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                      ; Disabilitazione delle interruzioni
                      cli

                      ; salvataggio dei registri selettori
                      mov       ax, seg save_ds
                      mov       bx, gs
                      mov       gs, ax
                      mov       ax, ds
                      mov       [gs:save_ds], ax
                      mov       ax, ss
                      mov       [gs:save_ss], ax
		      mov	[gs:save_esp], esp

                      ; Abilitazione linea A20
                      call      enable_a20

                      ; Caricamento della GDT e della IDT
                      xor       eax, eax
                      mov       eax, seg gdt
                      mov       es, ax
                      shl       eax, 4
                      add       eax, gdt
                      mov       [es:gdt_base], eax              ; Inizializzazione del campo base del descrittore
                      lgdt      [es:gdtr]                       ; Caricamento del descrittore in GDTR
                      lidt      [es:idtr]


                      ; Caricamento dei parametri dalla pila (e conversione in indirizzi lineari)
                      xor       eax, eax
                      xor       ebx, ebx
                      xor       ecx, ecx
                      xor       edx, edx

                      mov       dx, ds                          ; selettore
                      shl       edx, 4                          ; selettore*16

                      mov       ax, [bp+4]                      ; offset di &kernel
                      mov       bx, [bp+6]                      ; offset di &mbi
                      mov       cx, [bp+8]                      ; offset di &bldr32

                      add       eax, edx                        ; indirizzo lineare &kernel in EAX
                      add       ebx, edx                        ; indirizzo lineare &mbi in EBX
                      add       ecx, edx                        ; indirizzo lineare &bldr32 in ECX

                      ; Scrittura dell'entry-point (indirizzo lineare) all'indirizzo dell'istruzione
                      ; "jmp far" che salta al codice a 32 bit (vedere pi avanti)
                      mov       dx, seg start32
                      mov       ds, dx
                      mov       [ds:start32], ecx

                      

                      ; Abilitazione della modalit protetta
.set_pmode:           smsw      dx                              ; mov edx, cr0
                      or        dl, 1
                      lmsw      dx                              ; mov cr0, edx


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;; Altre operazioni di impostazione dello stato della macchina ;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                      ;;;  Scrivo in cx lo stack segment da ripristinare al ritorno da nucleo
                      xor       ecx, ecx
                      mov         cx, ss
                      ;;;
                      ; Inizializzazione dei registri selettore
                      xor       edx, edx
                      mov       dx, ss
                      shl       edx, 4
                      add       esp, edx                        ; top dello stack

                      mov       dx, 10h
                      mov       ds, dx
                      mov       es, dx
                      mov       fs, dx
                      mov       gs, dx
                      mov       ss, dx


                      ; Salto alla parte a 32 bit (equivale a "jmp far [selettoreGDT:offset32]")
                      db       66h
                      db       67h
                      db       09Ah                             ; call far
start32:              dd       00000000h                        ; offset
                      dw       0008h                            ; selettore
    


 					;Carico i registri di segmento SS, DS, ES, FS e GS necessari nella modalit reale
 
    				  xor ax, ax
    				  mov al, 10h
    				  mov ds, ax
    				  mov es, ax
    				  mov fs, ax
    				  mov gs, ax
    				  mov ss, ax

    				  db  0fh, 20h, 0c0h
    				  db  24h, 0feh
    				  db  0fh, 22h, 0c0h

					 ;Carico una Interrupt descriptor table compatibile con la modalit reale
    				  mov ax, seg real_idtr
    				  mov gs, ax
    				  lidt [gs:real_idtr] 


    				  mov ax, seg save_ds
    				  mov gs, ax
    				  mov ax, [gs:save_ds]
    				  mov ds, ax
    				  mov ax, [gs:save_ss]
    				  mov ss, ax
    				  mov esp, [gs:save_esp]
					 ;Ripristino il CS della modalit reale con una jmp.
    				  jmp word seg qui:qui
qui:

    				  call disable_a20   ;Disabilito la linea A20

    				  xor   ebx, ebx
    				  xor   ecx, ecx
    				  xor   edx, edx
    				  xor   esi, esi
    				  xor   edi, edi

    				  pop   gs
    				  pop   fs
    				  pop   es
    				  pop   di
    				  pop   si
    				  pop   dx
    				  pop   cx
     				  pop   bx
    				  popf
    				  xor ax, ax 
    				  
    				  leave
    				  ret




; Funzione: _get_ds
; Restituisce il valore del registro DS
; Parametri: nessuno
; Risultato: nessuno
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

_get_ds:              xor       eax, eax
                      mov       ax, ds
                      ret


; INT 12 - Get memory size
; Ritorno: AX = KB di memoria contigui a partire dall'indirizzo assoluto 00000h
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

_get_mem_lower:       push      bp
                      mov       bp, sp

                      int       12h

                      leave
                      ret


; INT 15 - Get extended memory size
; TODO: commenti
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

_get_mem_upper:       push      bp
                      mov       bp, sp
                        sub     sp, 4

                        push    ebx
                        push    edi
                        push    ecx
                        mov     ax, es
                        push    ax

                        mov     eax, 0
                        mov     [bp-4], eax
                        mov     ebx, 0
.moremem:
                        mov     eax, 0E820h
                        mov     cx, seg mem_buf
                        mov     es, cx
                        mov     di, mem_buf
                        mov     ecx, mem_buf_end - mem_buf
                        mov     edx, 534D4150h
                        int     15h
                        jc      .error
                        cmp     eax, 534D4150h
                        jne     .error
                        cmp     ecx, 20
                        jl      .error
                        mov     eax, [es:type]
                        cmp     eax, 1
                        jne     .notavail
                        mov     eax, [es:baseAddrLow]
                        cmp     eax, 100000h
                        jb      .notavail
                        mov     eax, [es:lengthLow]
                        add     [bp-4], eax
.notavail:              cmp     ebx, 0
                        jne     .moremem

                        xor     eax, eax
                        xor     edx, edx
                        mov     ax, [bp-4]
                        mov     dx, [bp-2]
                        jmp     .return
.error
                        xor     eax, eax
.return:             
                        pop     cx
                        mov     es, cx
                        pop     ecx
                        pop     edi
                        pop     ebx

                        leave
                        ret

