/******************************************************************************
 * Boot Loader v1.0                                                           *
 * di Valerio Di Gregorio                                                     *
 *                                                                            *
 * Boot Loader v1.0 carica da shell di FreeDOS il "Nucleo di Calcolatori      *
 * Elettronici v1.0". Questo progetto  conforme alle "Multiboot              *
 * Specification version 0.6.95".                                             *
 ******************************************************************************/

typedef char byte;

/* Immagine del nucleo (rispecchia la struttura definita in coff16.c) */
struct ImgCoff_go32 {
    unsigned long n;
    unsigned long ds;
	unsigned long* size;
	unsigned long* vaddr;
	byte** segment;
	void* entry_point;	
};

/* Converte un indirizzo espesso come selettore:offset in indirizzo lineare */
void* vaddr(unsigned long segment, void* offset) {
   if(!offset) return 0;
   return (void*)((segment<<4) + (unsigned long)offset);
}

/* Effettua lo spostamento del nucleo agli indirizzi indicati all'interno dell'eseguibile formato COFF */
extern "C" void* move(ImgCoff_go32* kernel){

     void* v_tmp;
     unsigned long ul_tmp;
     
     byte* dst;                     // Indirizzo di destinazione
     byte* src;                     // Indirizzo sorgente
     unsigned long size;            // Dimensione del segmento da spostare
     unsigned long entry_point;     // Entry point del nucleo

     /* Traduzione degli indirizzi nella struttura in indirizzi lineari */
     v_tmp = vaddr(kernel->ds, (void*)kernel->size);
     kernel->size = (unsigned long*)v_tmp; 

     v_tmp = vaddr(kernel->ds, (void*)kernel->vaddr);
     kernel->vaddr = (unsigned long*)v_tmp;

     kernel->segment = (byte**)vaddr(kernel->ds, (void*)kernel->segment);

     for(int i=0; i<kernel->n; i++)
	 if (kernel->segment[i])
		 kernel->segment[i] = (byte*)vaddr(kernel->ds, (void*)kernel->segment[i]);
         
     kernel->entry_point = vaddr(kernel->ds, kernel->entry_point);

     /* Spostamento del nucleo */
     for(int i=0; i<kernel->n; i++) {

         dst = (byte*)kernel->vaddr[i];
         size = kernel->size[i];
         src = kernel->segment[i];
         
	 if (src) 
		 for (int j=0; j<size; j++) dst[j] = src[j];  // Spostamento di segmento byte a byte
	 else
		 for (int j=0; j<size; j++) dst[j] = 0;

         /* Posizionamento dell'entry point */
         if (kernel->entry_point >= src &&
             kernel->entry_point < src + size) {

             entry_point = (unsigned long)dst + ((unsigned long)kernel->entry_point - (unsigned long)src);
         }
     }

     /* Ritorno al chiamante comunicando l'entry point (indirizzo lineare) */
     return (void*)entry_point;
     
}
