# Boot Loader v1.0
# di Valerio Di Gregorio
#
# Boot Loader v1.0 carica da shell di FreeDOS il "Nucleo di Calcolatori
# Elettronici v1.0". Questo progetto Ë conforme alle "Multiboot
# Specification version 0.6.95".
########################################################################################
.text
.extern _move
.global _main, start

# Boot Loader 32 bits
########################################################################################

start:
_main:                pushl   %ebp
                      movl    %esp, %ebp
                                     
		      pushl   %ebx                 # Push di ebx = &mbi

                      pushl   %eax                 # Push di eax = &kernel
                                       
                      call    _move
                      movl    %eax, %edx           # risultato in &edx (entry point)
                      addl    $4, %esp             # Pop di &kernel

                      # Completamento dello stato della macchina secondo le specifiche Multiboot
                      pushf
                      popl    %eax
                      and     $0xFFFDFFFF, %eax    #VM = EFLAG[17] = 0
                      pushl   %eax
                      popf
                      movl    $0x2BADB002, %eax    # %eax = 0x2BADB002#
                      pop     %ebx                 # %ebx = &mbi#
                      call        *%edx
#######################################################################################

                      # bldr32 non viene caricato al suo indirizzo di collegamento,
                      # ma ad un indirizzo qualunque. Dobbiamo sapere a che
                      # indirizzo e' stato caricato
                      call 1f         # salviamo in pila %eip
1:                    popl %esi       # %esi contiene l'indirizzo della popl
                      subl $1b, %esi  # sottraiamo l'indirizzo di collegamento
                                      # della stessa istruzione, ottenendo l'indirizzo
                                      # di caricamento di bldr32

		      #Carica una gdt che emula la modalità reale
                      addl %esi, (real_pgdt_desc+2)(%esi)
                      lgdt real_pgdt_desc(%esi)

		      popl %ebp
		      popl %ebx
		      popl %eax
		      shll $4, %eax
		      addl %eax, %ebx

                      # Carico il nuovo cs eseguendo un jmp far
                      pushl $0x08
                      movl $salta, %eax
                      addl %esi, %eax
                      pushl %eax
                      lret
salta:

 
		      .byte 0x66
		      jmp *%ebx 
########################################################################################


#GDT 
.equ    GDTFLAG_G_4KB , 0x80 
.equ    GDTFLAG_DB_32 , 0x40 
.equ    GDTFLAG_P, 0x80 
.equ    GDTFLAG_DPL_0 , 0 
.equ    GDTFLAG_S, 0x10 
.equ    GDTFLAG_TYPE_CODE, 0x0A 
.equ    GDTFLAG_TYPE_DATA, 0x02 


real_gdt:

        .long 0
        .long 0

        .word 0xFFFF    #limit 0-15 
        .word 0x0000    #base 0-15 
        .byte 0x00    #base 16-23 
        .byte GDTFLAG_P+GDTFLAG_S+GDTFLAG_TYPE_CODE 
        .byte 0x0F
        .byte 0x00    #base 24-31 

        .word 0xFFFF    #limit 0-15 
        .word 0x0000    #base 0-15 
        .byte 0x00    #base 16-23 
        .byte GDTFLAG_P+GDTFLAG_S+GDTFLAG_TYPE_DATA 
        .byte 0x0F
        .byte 0x00    #base 24-31 


real_pgdt_desc:
        .word 3*8-1
        .long real_gdt

